local require = GLOBAL.require

local RecipePopup = require "widgets/recipepopup"
local ImageButton = require "widgets/imagebutton"
local MyRecipePopup = require "widgets/myrecipepopup"

require "simutil"
require "constants"

-- Get recipe popup class refresh function
local RecipePopup_Refresh_base = RecipePopup.Refresh or function() print("ERROR: Can't run RecipePopup_Refresh_base!") return "" end

-- Function to create a button, return pointer to button
local function ButtonCreate(caption, x, y, image_scale_x, image_scale_y, parent, onClick_function)
    -- Create image button and put in in the parent/container
    local new_button = parent:AddChild(ImageButton())
    -- Set image scale
    new_button.image:SetScale(image_scale_x,image_scale_y)
    -- Set position in parent
    new_button:SetPosition(x,y,0)
    -- Set onClick function
    new_button:SetOnClick(onClick_function)
    -- Show button
    new_button:Show()
    -- Set button text
    new_button:SetText(caption)
    -- Enable for click
    new_button:Enable()
    -- Return new button
    return new_button
end

local function ButtonExists(button_pointer)
    -- Check if button exists, or parent for button was destroyed and need to be re-created
    if button_pointer==nil or button_pointer.parent==nil then
        -- button or parent don't exist
        return false
    else
        -- button exsits
        return true
    end
end

local function WidgetPosition(widget_to_set, x_align, y_align, x_marginal, y_marginal, controls, screensize)
    -- init vars
    local dir_x=0
    local dir_y=0
    local anchor_x=0
    local anchor_y=0
    local margin_dir_x=0
    local margin_dir_y=0
    -- Set x settings
    if x_align=="left" then
        dir_x=-1
        anchor_x=1
        margin_dir_x=1
    elseif x_align=="center" then
        -- All 0, default
    elseif x_align=="right" then
        dir_x=1
        anchor_x=-1
        margin_dir_x=-1
    end
    -- Set y settings
    if y_align=="top" then
        anchor_y=-1
        margin_dir_y=-1
    elseif y_align=="middle" then
        dir_y=-1
    elseif y_align=="bottom" then
        dir_y=-2
        anchor_y=1
        margin_dir_y=1
    end
    -- Get scale
    local hudscale=controls.top_root:GetScale()
    local screenw=screensize[1]/hudscale.x
    local screenh=screensize[2]/hudscale.y
    -- Set position
    widget_to_set:SetPosition((anchor_x*x_marginal)+(dir_x*screenw/2)+(margin_dir_x*x_marginal),(anchor_y*y_marginal)+(dir_y*screenh/2)+(margin_dir_y*y_marginal),0)
end

local function WidgetAdd(WidgetClass,x_align,y_align,x_marginal,y_marginal,hide,controls_set_function)
    -- Add post script to controls construction
    AddClassPostConstruct("widgets/controls",function(controls)
        -- Execute task in controls class instance in next tick
        controls.inst:DoTaskInTime(0,function()
            -- Create new widget in controls
            local new_widget=controls.top_root:AddChild(WidgetClass())
            -- Let us do some code with the new widget
            controls_set_function(controls,new_widget)
            -- Get screen size and HUD scale
            local screensize={GLOBAL.TheSim:GetScreenSize()}
            -- Place widget where we want it
            WidgetPosition(new_widget,x_align,y_align,x_marginal,y_marginal,controls,screensize)
            -- Get original onUpdate function so we can call it in our custom onUpdate function, so we don't lose the original
            local OnUpdate_base=controls.OnUpdate
            -- Replace onUpdate function with our special one so our own widget update along
            controls.OnUpdate=function(self, dt)
                -- Call base
                OnUpdate_base(self,dt)
                -- Get current screen size
                local curscreensize={GLOBAL.TheSim:GetScreenSize()}
                if curscreensize[1]~=screensize[1] or curscreensize[2]~=screensize[2] then
                    -- Save to keep track of changes
                    screensize[1]=curscreensize[1]
                    screensize[2]=curscreensize[2]
                    WidgetPosition(new_widget,x_align,y_align,x_marginal,y_marginal,controls,curscreensize)
                end
            end

            -- Hide widget on create
            if hide==true then
                new_widget:Hide()
            end

        end)
    end)
end

-- Function for inventory stock change
local function Listeners_inventory_stock_change(function_to_call)
    -- Add listeners to events that cause inventory stock change
    GLOBAL.ThePlayer:ListenForEvent("itemget",function_to_call)
    GLOBAL.ThePlayer:ListenForEvent("itemlose",function_to_call)
    GLOBAL.ThePlayer:ListenForEvent("stacksizechange",function_to_call)
end

-- Hook onto RecipePopup refresh function, overide it
function RecipePopup:Refresh()
    -- Run recipe refresh
    RecipePopup_Refresh_base(self)
    -- Get recipe
    local current_recipe=self.name:GetString()
    -- Set offset
    local x_offset=196
    -- Show refreshed recipe popup name
    if ButtonExists(self.fav_button_1)==false then
        -- Create button in recipe popup and add it to self, for future reference
        -- Set position depending on skin
        if self.skin_options==1 then
            self.fav_button_1=ButtonCreate("1",x_offset,-200,0.2,0.75,self.contents,function() GLOBAL.ThePlayer.HUD.controls.fav_1:LoadRecipe(self.recipe, self.owner) end)
        else
            self.fav_button_1=ButtonCreate("1",x_offset,-220,0.2,0.75,self.contents,function() GLOBAL.ThePlayer.HUD.controls.fav_1:LoadRecipe(self.recipe, self.owner) end)
        end
    end
    if ButtonExists(self.fav_button_2)==false then
        -- Create button in recipe popup and add it to self, for future reference
        -- Set position depending on skin
        if self.skin_options==1 then
            self.fav_button_2=ButtonCreate("2",60+x_offset,-200,0.2,0.75,self.contents,function() GLOBAL.ThePlayer.HUD.controls.fav_2:LoadRecipe(self.recipe, self.owner) end)
        else
            self.fav_button_2=ButtonCreate("2",60+x_offset,-220,0.2,0.75,self.contents,function() GLOBAL.ThePlayer.HUD.controls.fav_2:LoadRecipe(self.recipe, self.owner) end)
        end
    end   
    if ButtonExists(self.fav_button_3)==false then
        -- Create button in recipe popup and add it to self, for future reference
        -- Set position depending on skin
        if self.skin_options==1 then
            self.fav_button_3=ButtonCreate("3",120+x_offset,-200,0.2,0.75,self.contents,function() GLOBAL.ThePlayer.HUD.controls.fav_3:LoadRecipe(self.recipe, self.owner) end)
        else
            self.fav_button_3=ButtonCreate("3",120+x_offset,-220,0.2,0.75,self.contents,function() GLOBAL.ThePlayer.HUD.controls.fav_3:LoadRecipe(self.recipe, self.owner) end)
        end
    end  
    if ButtonExists(self.fav_button_4)==false then
        -- Create button in recipe popup and add it to self, for future reference
        -- Set position depending on skin
        if self.skin_options==1 then
            self.fav_button_4=ButtonCreate("4",180+x_offset,-200,0.2,0.75,self.contents,function() GLOBAL.ThePlayer.HUD.controls.fav_4:LoadRecipe(self.recipe, self.owner) end)
        else
            self.fav_button_4=ButtonCreate("4",180+x_offset,-220,0.2,0.75,self.contents,function() GLOBAL.ThePlayer.HUD.controls.fav_4:LoadRecipe(self.recipe, self.owner) end)
        end
    end  
    if ButtonExists(self.fav_button_5)==false then
        -- Create button in recipe popup and add it to self, for future reference
        -- Set position depending on skin
        if self.skin_options==1 then
            self.fav_button_5=ButtonCreate("5",240+x_offset,-200,0.2,0.75,self.contents,function() GLOBAL.ThePlayer.HUD.controls.fav_5:LoadRecipe(self.recipe, self.owner) end)
        else
            self.fav_button_5=ButtonCreate("5",240+x_offset,-220,0.2,0.75,self.contents,function() GLOBAL.ThePlayer.HUD.controls.fav_5:LoadRecipe(self.recipe, self.owner) end)
        end
    end               
end

-- Add my widget
WidgetAdd(MyRecipePopup, "left", "top", GetModConfigData("position"), 20, true, function(controls, new_widget)
    -- Save in controls
    controls.fav_1=new_widget
    -- Add function to inventory stock change
    Listeners_inventory_stock_change(function() new_widget:Refresh() end)
end)
WidgetAdd(MyRecipePopup, "left", "top", GetModConfigData("position")+(90*1), 20, true, function(controls, new_widget)
    -- Save in controls
    controls.fav_2=new_widget
    -- Add function to inventory stock change
    Listeners_inventory_stock_change(function() new_widget:Refresh() end)
end)
WidgetAdd(MyRecipePopup, "left", "top", GetModConfigData("position")+(90*2), 20, true, function(controls, new_widget)
    -- Save in controls
    controls.fav_3=new_widget
    -- Add function to inventory stock change
    Listeners_inventory_stock_change(function() new_widget:Refresh() end)
end)
WidgetAdd(MyRecipePopup, "left", "top", GetModConfigData("position")+(90*3), 20, true, function(controls, new_widget)
    -- Save in controls
    controls.fav_4=new_widget
    -- Add function to inventory stock change
    Listeners_inventory_stock_change(function() new_widget:Refresh() end)
end)
WidgetAdd(MyRecipePopup, "left", "top", GetModConfigData("position")+(90*4), 20, true, function(controls, new_widget)
    -- Save in controls
    controls.fav_5=new_widget
    -- Add function to inventory stock change
    Listeners_inventory_stock_change(function() new_widget:Refresh() end)
end)